@@ -92,7 +92,9 @@ class ISBNMerge(object):
         self.isbnless_results = []
         self.results = []
         self.log = log
-        self.use_xisbn = True
+        # The xISBN service has been de-commissioned
+        # https://www.oclc.org/developer/news/2018/xid-decommission.en.html
+        self.use_xisbn = False
 
     def isbn_in_pool(self, isbn):
         if isbn:
@@ -493,8 +495,7 @@ def identify(log, abort,  # {{{
 
     log('The identify phase took %.2f seconds'%(time.time() - start_time))
     log('The longest time (%f) was taken by:'%longest, lp)
-    log('Merging results from different sources and finding earliest ',
-            'publication dates from the worldcat.org service')
+    log('Merging results from different sources')
     start_time = time.time()
     results = merge_identify_results(results, log)
 
